# Jannis' Superkaramba python library
# (c) 2003 J@hacked.it <www.jannis.to/daily/>
# Fell free to use :)
import karamba
import math
import string
import re

# Vars
popupSteps = 3

# Creates a box of the desired size using the passed image
# files
boxes = {}
def createBox(widget, name,  x, y, w, h, path, animate):

	# First create the images
	tl  = karamba.createImage(widget, 0, 0, path+"tl.png")
	t   = karamba.createImage(widget, 0, 0, path+"t.png")
	tr  = karamba.createImage(widget, 0, 0, path+"tr.png")
	l   = karamba.createImage(widget, 0, 0, path+"l.png")
	c   = karamba.createImage(widget, 0, 0, path+"c.png")
	r   = karamba.createImage(widget, 0, 0, path+"r.png")
	bl  = karamba.createImage(widget, 0, 0, path+"bl.png")
	b   = karamba.createImage(widget, 0, 0, path+"b.png")
	br  = karamba.createImage(widget, 0, 0, path+"br.png")

	# Now save it
	boxes[name] = {"x"        : x,
				   "y"        : y,
				   "h"        : h,
				   "w"        : w,
				   "path"     : path,
				   "images"   : {"tl" : tl, "t" : t, "tr" : tr, "l" : l, "c" : c,
				                  "r" : r, "bl" : bl, "b" : b, "br" :  br
								},
				   "imagePos" : {"tl": [0, 0], "t": [0, 0], "tr": [0, 0],
				                 "l": [0, 0],  "c": [0, 0], "r": [0, 0],
				                 "bl": [0, 0], "b": [0, 0], "br": [0, 0]
				  				},
				   "imageDim" : {"tl": [karamba.getImageWidth(widget, tl),
				   					   karamba.getImageHeight(widget, tl)],
								 "tr": [karamba.getImageWidth(widget, tr),
				   					   karamba.getImageHeight(widget, tr)],
								 "bl": [karamba.getImageWidth(widget, bl),
				   					   karamba.getImageHeight(widget, bl)],
								 "br": [karamba.getImageWidth(widget, br),
				   					   karamba.getImageHeight(widget, br)],
								}

				 }

	# And make it the right size
	if animate == False:
		resizeBox(widget, name, w, h)
	else:
		popupBox(widget, name, w, h)

# Sizes a box
def resizeBox (widget, boxName, w, h):
	x    = boxes[boxName]['x']
	y    = boxes[boxName]['y']

	tlX = x
	tlY = y
	tX  = (x+boxes[boxName]['imageDim']['tl'][0])
	tY  = y
	trX = (x+w-boxes[boxName]['imageDim']['tr'][0])
	trY = y
	lX  = x
	lY  = (y+boxes[boxName]['imageDim']['tl'][1])
	cX  = (x+boxes[boxName]['imageDim']['tl'][0])
	cY  = (y+boxes[boxName]['imageDim']['tl'][1])
	rX  = (x+w-boxes[boxName]['imageDim']['tr'][0])
	rY  = (y+boxes[boxName]['imageDim']['tr'][1])
	blX = x
	blY = (y+h-boxes[boxName]['imageDim']['bl'][1])
	bX  = (x+boxes[boxName]['imageDim']['bl'][0])
	bY  = (y+h-boxes[boxName]['imageDim']['bl'][1])
	brX = (x+w-boxes[boxName]['imageDim']['br'][0])
	brY = (y+h-boxes[boxName]['imageDim']['br'][1])

	# Make them the right size
	karamba.resizeImage(widget, boxes[boxName]['images']['t'],
	                    (w-boxes[boxName]['imageDim']['tl'][0]-boxes[boxName]['imageDim']['tr'][0]),
						boxes[boxName]['imageDim']['tl'][1])
	karamba.resizeImage(widget, boxes[boxName]['images']['b'],
	                    (w-boxes[boxName]['imageDim']['bl'][0]-boxes[boxName]['imageDim']['br'][0]),
						boxes[boxName]['imageDim']['bl'][1])
	karamba.resizeImage(widget, boxes[boxName]['images']['l'], boxes[boxName]['imageDim']['tl'][0],
						(h-boxes[boxName]['imageDim']['tl'][1]-boxes[boxName]['imageDim']['bl'][1]))
	karamba.resizeImage(widget, boxes[boxName]['images']['r'], boxes[boxName]['imageDim']['tr'][0],
						(h-boxes[boxName]['imageDim']['tr'][1])-boxes[boxName]['imageDim']['br'][1])
	karamba.resizeImage(widget, boxes[boxName]['images']['c'], (w-boxes[boxName]['imageDim']['tl'][0]-boxes[boxName]['imageDim']['tr'][0]),
						(h-boxes[boxName]['imageDim']['tl'][1])-boxes[boxName]['imageDim']['bl'][1])

	# Move them to the right position, too :)
	karamba.moveImage(widget, boxes[boxName]['images']['tl'], tlX, tlY)
	karamba.moveImage(widget, boxes[boxName]['images']['t'], tX,  tY)
	karamba.moveImage(widget, boxes[boxName]['images']['tr'], trX, trY)
	karamba.moveImage(widget, boxes[boxName]['images']['l'], lX,  lY)
	karamba.moveImage(widget, boxes[boxName]['images']['c'], cX,  cY)
	karamba.moveImage(widget, boxes[boxName]['images']['r'], rX,  rY)
	karamba.moveImage(widget, boxes[boxName]['images']['bl'], blX, blY)
	karamba.moveImage(widget, boxes[boxName]['images']['b'], bX,  bY)
	karamba.moveImage(widget, boxes[boxName]['images']['br'], brX, brY)
	karamba.redrawWidget(widget)

	# Now save it
	boxes[boxName]['h']        = h
	boxes[boxName]['w']        = w
	boxes[boxName]['imagePos'] = {"tl": [tlX, tlY], "t": [tX, tY], "tr": [trX, trY],
				                 "l": [lX, lY],     "c": [cX, cY], "r": [rX, rY],
				                 "bl": [blX, blY], "b": [bX, bY], "br": [brX, brY]}


# Moves a box to a total position
def moveBoxTo(widget, boxName, x, y, animate):
	moveBoxBy(widget, boxName, (x-boxes[boxName]['x']), (y-boxes[boxName]['y']), animate)

# Moves a box relatively
def moveBoxBy(widget, boxName, x, y, animate):
	karamba.moveImage(widget, boxes[boxName]['images']['tl'], (boxes[boxName]['imagePos']['tl'][0]+x),
					  (boxes[boxName]['imagePos']['tl'][1]+y))
	karamba.moveImage(widget, boxes[boxName]['images']['t'], (boxes[boxName]['imagePos']['t'][0]+x),
					  (boxes[boxName]['imagePos']['t'][1]+y))
	karamba.moveImage(widget, boxes[boxName]['images']['tr'], (boxes[boxName]['imagePos']['tr'][0]+x),
					  (boxes[boxName]['imagePos']['tr'][1]+y))
	karamba.moveImage(widget, boxes[boxName]['images']['l'], (boxes[boxName]['imagePos']['l'][0]+x),
					  (boxes[boxName]['imagePos']['l'][1]+y))
	karamba.moveImage(widget, boxes[boxName]['images']['c'], (boxes[boxName]['imagePos']['c'][0]+x),
					  (boxes[boxName]['imagePos']['c'][1]+y))
	karamba.moveImage(widget, boxes[boxName]['images']['r'], (boxes[boxName]['imagePos']['r'][0]+x),
					  (boxes[boxName]['imagePos']['r'][1]+y))
	karamba.moveImage(widget, boxes[boxName]['images']['bl'], (boxes[boxName]['imagePos']['bl'][0]+x),
					  (boxes[boxName]['imagePos']['bl'][1]+y))
	karamba.moveImage(widget, boxes[boxName]['images']['b'], (boxes[boxName]['imagePos']['b'][0]+x),
					  (boxes[boxName]['imagePos']['b'][1]+y))
	karamba.moveImage(widget, boxes[boxName]['images']['br'], (boxes[boxName]['imagePos']['br'][0]+x),
					  (boxes[boxName]['imagePos']['br'][1]+y))
	karamba.redrawWidget(widget)

# Pops up an existing box
def popupBox(widget, boxName, w, h):
	# calculate the steps to resize the box by it's minimal size
	curW = boxes[boxName]['imageDim']['tl'][0]+boxes[boxName]['imageDim']['tr'][0]
	curH = boxes[boxName]['imageDim']['tl'][1]+boxes[boxName]['imageDim']['bl'][1]

	# steps to resize
	difW   = w - curW
	difH   = h - curH
	stepW  = math.floor(difW/popupSteps)
	stepH  = math.floor(difH/popupSteps)

	# center of the box
	bX = boxes[boxName]['x']
	bY = boxes[boxName]['y']
	cX = bX+math.floor(w/2)
	cY = bY+math.floor(h/2)

	# resize!
	i = 0
	while i < popupSteps:
		boxes[boxName]['x'] = cX - round(curW/2)
		boxes[boxName]['y'] = cY - round(curH/2)
		resizeBox(widget, boxName, curW, curH)
		curW+=stepW
		curH+=stepH
		i+=1
	if curW < w or curH < h:
		boxes[boxName]['x'] = bX
		boxes[boxName]['y'] = bY
		resizeBox(widget, boxName, w, h)

# The aquivalent to popUpBox, in the reverse direction
def popDownBox(widget, boxName):
	minimizeBox(widget, boxName)

#minimizes a box
def minimizeBox(widget, boxName):
	resizeBox(widget, boxName, boxes[boxName]['imageDim']['tl'][0]+boxes[boxName]['imageDim']['tr'][0],
			  boxes[boxName]['imageDim']['tl'][1]+boxes[boxName]['imageDim']['bl'][1])

# Hides a box
def hideBox(widget, boxName):
	karamba.hideImage(widget, boxes[boxName]['images']['tl'])
	karamba.hideImage(widget, boxes[boxName]['images']['t'])
	karamba.hideImage(widget, boxes[boxName]['images']['tr'])
	karamba.hideImage(widget, boxes[boxName]['images']['l'])
	karamba.hideImage(widget, boxes[boxName]['images']['c'])
	karamba.hideImage(widget, boxes[boxName]['images']['r'])
	karamba.hideImage(widget, boxes[boxName]['images']['bl'])
	karamba.hideImage(widget, boxes[boxName]['images']['b'])
	karamba.hideImage(widget, boxes[boxName]['images']['br'])
	karamba.redrawWidget(widget)

# Shows a box
def showBox(widget, boxName):
	karamba.showImage(widget, boxes[boxName]['images']['tl'])
	karamba.showImage(widget, boxes[boxName]['images']['t'])
	karamba.showImage(widget, boxes[boxName]['images']['tr'])
	karamba.showImage(widget, boxes[boxName]['images']['l'])
	karamba.showImage(widget, boxes[boxName]['images']['c'])
	karamba.showImage(widget, boxes[boxName]['images']['r'])
	karamba.showImage(widget, boxes[boxName]['images']['bl'])
	karamba.showImage(widget, boxes[boxName]['images']['b'])
	karamba.showImage(widget, boxes[boxName]['images']['br'])
	karamba.redrawWidget(widget)

# Returns a box for the user to play with
def getBox(widget, boxName):
	return boxes[boxName]

# Since karamba.createRichText crashes, we need to use the simple textArea
def unHTML(source):

	# Replace known tags
	tags = [['<br />','\n'], ['<br>','\n'], ['<p>', '\n\n']]
	for tag in tags:
		source = string.replace(source, tag[0], tag[1])

	# Eliminate the rest of the tags
	pattern = re.compile(r'<([^>]*)>', re.MULTILINE)

	# Return result
	return (pattern.sub('', source, 0))